package com.ruoyi.project.pssr.mapper;

import java.util.List;
import com.ruoyi.framework.aspectj.lang.annotation.DataScope;
import com.ruoyi.project.pssr.domain.TPssrPower;

/**
 * 临时电源Mapper接口
 *
 * @author ssy
 * @date 2024-11-01
 */
public interface TPssrPowerMapper
{
    /**
     * 查询临时电源
     *
     * @param id 临时电源ID
     * @return 临时电源
     */
    public TPssrPower selectTPssrPowerById(Long id);
    public int selectNoConfirmCount(Long subId);

    /**
     * 查询临时电源列表
     *
     * @param tPssrPower 临时电源
     * @return 临时电源集合
     */
    
    public List<TPssrPower> selectTPssrPowerList(TPssrPower tPssrPower);

    /**
     * 新增临时电源
     *
     * @param tPssrPower 临时电源
     * @return 结果
     */
    public int insertTPssrPower(TPssrPower tPssrPower);

    /**
     * 修改临时电源
     *
     * @param tPssrPower 临时电源
     * @return 结果
     */
    public int updateTPssrPower(TPssrPower tPssrPower);
    public int updateTPssrPowerByIds(TPssrPower tPssrPower);



    /**
     * 删除临时电源
     *
     * @param id 临时电源ID
     * @return 结果
     */
    public int deleteTPssrPowerById(Long id);

    /**
     * 批量删除临时电源
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    public int deleteTPssrPowerByIds(Long[] ids);
}
